@@ -29,7 +29,7 @@ int l2tp_wan_init();
 int l2tp_wan_dispose();
 int l2tp_wan_attach(void *rfc, struct ppp_link **link);
 void l2tp_wan_detach(struct ppp_link *link);
-int l2tp_wan_input(struct ppp_link *link, struct mbuf *m);
+int l2tp_wan_input(struct ppp_link *link, mbuf_t m);
 void l2tp_wan_xmit_full(struct ppp_link *link);
 void l2tp_wan_xmit_ok(struct ppp_link *link);
 void l2tp_wan_input_error(struct ppp_link *);
